\version "2.20.0"

preferBreakAfter = #(define-music-function (w m) (integer? ly:music?) 
#{
  \override Score.NonMusicalPaperColumn.line-break-penalty = #(- w)
  #m
  \revert Score.NonMusicalPaperColumn.line-break-penalty
#})

loose = #(define-scheme-function (d) (rational?)
(if final
  #{
    \layout {
      \context {
        \Score
        \override SpacingSpanner.base-shortest-duration = #(ly:make-moment d)
      }
    }
  #}))

spacer = #(define-music-function (upbeat m) (ly:music? ly:music?)
  (let* ((core  (ly:moment-sub (ly:music-length m) (ly:music-length upbeat)))
         (num   (ly:moment-main-numerator core))
         (denom (ly:moment-main-denominator core)))
    #{ #upbeat \set Timing.measurePosition = #(ly:make-moment 0) \scaleDurations #(cons num denom) s1 #} 
  ))

#(define (incrementBarNumber context)
  (let* ((score (ly:context-find context 'Score)))
    (ly:context-set-property! score 'currentBarNumber 
      (1+ (ly:context-property score 'currentBarNumber)))))

#(define (decrementBarNumber context)
  (let* ((score (ly:context-find context 'Score)))
    (ly:context-set-property! score 'currentBarNumber 
      (1- (ly:context-property score 'currentBarNumber)))))

plugins = \with {
  \consists "Timing_translator"
  \consists "Default_bar_line_engraver"
  \consists "Repeat_acknowledge_engraver"
}

\layout {
  staffspace = #(ly:output-def-lookup $defaultpaper 'staff-space)
  indent = #(* 24 staffspace)
  \override AccidentalSuggestion.avoid-slur = #'outside
  \override DynamicTextSpanner.style = #'none
  \override LaissezVibrerTie.details.note-head-gap = -2
  \override LaissezVibrerTie.extra-offset = #'(2 . 0)
  \override LaissezVibrerTie.X-extent = #'(0 . 0)

  \context { \Score barNumberVisibility = #all-bar-numbers-visible }
  \context { \StaffGroup \override SystemStartBracket.collapse-height = 0 }
  \context { \Staff printKeyCancellation = ##f }
  \context {
    \Voice
    \override Script.script-priority = 190 % move trills inside partcombine texts
    \override DynamicText.extra-spacing-width = #'(+inf.0 . -inf.0)
    \hide TupletBracket
  }

  % Groups
  % Create StaffGroup contexts
  $@(map groupcxt groups)

  % Add details
  \context {
    \Violins
    instrumentName = "Violins"
    shortInstrumentName = "Vln"
  }
  \context {
    \Violas
    instrumentName = "Violas"
    shortInstrumentName = "Vla"
  }
  \context {
    \Oboes
    instrumentName = "Oboes"
    shortInstrumentName = "Ob"
  }
  \context {
    \Horns
    shortInstrumentName = "Hn"
  }

  % Instruments pairs
  % Create Staff contexts
  $@(map paircxt pairs)

  % Add details
  \context {
    \ViolaIandII
    instrumentName = \markup \pairinsts "I" "II" "Violas"
    shortInstrumentName = "Vla"
    aDueText = "unis"
    #(make-clef-set "alto")
  }
  \context {
    \FluteIandII
    instrumentName = \markup \pairinsts "I" "II" "Flutes"
    shortInstrumentName = "Fl"
    #(make-clef-set "treble")
    \consists \Print_directions
  }
  \context {
    \OboeIandII
    instrumentName = \markup \pairinsts "I" "II" "Oboes"
    shortInstrumentName = "Ob"
    #(make-clef-set "treble")
    \consists \Print_directions
  }
  \context {
    \CorAnglaisIandII
    instrumentName = \markup \pairinsts "I" "II" "Cors Anglais"
    shortInstrumentName = "CorA"
    instrumentTransposition = f
    #(make-clef-set "treble")
    \autoTranspose
    \consists \Print_directions
  }
  \context {
    \ClarinetIandII
    instrumentName = "Clarinets"
    shortInstrumentName = "Cl"
    #(make-clef-set "treble")
    \autoTranspose
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }
  \context {
    \BassoonIandII
    instrumentName = \markup \pairinsts "I" "II" "Bassoons"
    shortInstrumentName = "Bn"
    #(make-clef-set "bass")
    \consists \Print_directions
  }
  \context {
    \TrumpetIandII
    instrumentName = "Trumpets"
    shortInstrumentName = "Tpt"
    #(make-clef-set "treble")
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }
  \context {
    \HornIandII
    instrumentName = "Horns"
    shortInstrumentName = "Hn"
    #(make-clef-set "treble")
    \hide ClefModifier
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }
  \context {
    \HornIIIandIV
    instrumentName = "Horns"
    shortInstrumentName = "Hn"
    #(make-clef-set "treble")
    \hide ClefModifier
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }

  % Single instruments
  % Create Staff contexts
  $@(map staffcxt instruments)

  % Add details
  \context {
    \Flute
    instrumentName = "Flute"
    shortInstrumentName = "Fl"
    #(make-clef-set "treble")
  }
  \context {
    \Bassoon
    instrumentName = "Bassoon"
    shortInstrumentName = "Bn"
    #(make-clef-set "bass")
  }
  \context {
    \Violin
    \name "ViolinSolo"
    instrumentName = "Violin Solo"
    shortInstrumentName = "VlnS"
    #(make-clef-set "treble")
  }
  \context {
    \Viola
    instrumentName = "Viola"
    shortInstrumentName = "Vla"
    #(make-clef-set "alto")
  }
  \context {
    \Cello
    instrumentName = "Cello"
    shortInstrumentName = "Vlc"
    #(make-clef-set "bass")
  }
  \context {
    \Cello
    \name "CelloSolo"
    instrumentName = "Cello Solo"
    shortInstrumentName = "VlcS"
    #(make-clef-set "bass")
  }
  \context {
    \Bass
    instrumentName = "Bass"
    shortInstrumentName = "Db"
    instrumentTransposition = c
    #(make-clef-set "bass")
    \autoTranspose
  }
  \context {
    \Bass
    \name "BassSolo"
    instrumentName = "Violone Solo"
    shortInstrumentName = "Vne"
  }
  \context {
    \CB
    instrumentName = "Cello, Bass"
    shortInstrumentName = \markup \center-column { "Vlc" "Db" }
    #(make-clef-set "bass")
    aDueText = "Tutti"
    soloText = "Vlc"
    soloIIText = "Db"
  }
  \context {
    \CBB
    instrumentName = \markup \center-column { "Cello, Bass" "Bassoon" }
    shortInstrumentName = \markup \center-column { "Vlc" "Db Bn" }
    #(make-clef-set "bass")
    aDueText = "Tutti"
    soloText = "Vlc"
    soloIIText = "Db Bn"
  }
  \context {
    \Timpani
    instrumentName = "Timpani"
    shortInstrumentName = "Timp"
    #(make-clef-set "bass")
    \remove "Key_engraver"
    \consists \instrumentTuning
  }
  \context {
    \Trumpet
    instrumentName = "Trumpet"
    shortInstrumentName = "Tpt"
    #(make-clef-set "treble")
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
  }

  % First, second etc.
  \context { \OboeI    instrumentName=\markup \oneinst "I"   #(make-clef-set "treble") }
  \context { \OboeII   instrumentName=\markup \oneinst "II"  #(make-clef-set "treble") }
  \context { \HornI    instrumentName=\markup \oneinst "I"   #(make-clef-set "treble")
                       \hide ClefModifier \autoTranspose \remove "Key_engraver" }
  \context { \HornII   instrumentName=\markup \oneinst "II"  #(make-clef-set "treble")
                       \hide ClefModifier \autoTranspose \remove "Key_engraver" }
  \context { \HornIII  instrumentName=\markup \oneinst "III" #(make-clef-set "treble")
                       \hide ClefModifier \autoTranspose \remove "Key_engraver" }
  \context { \HornIV   instrumentName=\markup \oneinst "IV"  #(make-clef-set "treble")
                       \hide ClefModifier \autoTranspose \remove "Key_engraver" }
  \context { \ViolinI  instrumentName=\markup \oneinst "I"   #(make-clef-set "treble")
                       shortInstrumentName=\markup \hcenter-in #2 "I"  }
  \context { \ViolinII instrumentName=\markup \oneinst "II"  #(make-clef-set "treble")
                       shortInstrumentName=\markup \hcenter-in #2 "II" }
  \context { \ViolaI   instrumentName=\markup \oneinst "I"   #(make-clef-set "alto")
                       shortInstrumentName=\markup \hcenter-in #2 "I"  }
  \context { \ViolaII  instrumentName=\markup \oneinst "II"  #(make-clef-set "alto")
                       shortInstrumentName=\markup \hcenter-in #2 "II" }
}

#(hierarchy $defaultlayout)
